<?php
/**
 * The style component
 *
 * @package   BlockEnhancements
 * @author    Phi Phan <mrphipv@gmail.com>
 * @copyright Copyright (c) 2022, Phi Phan
 */

namespace BlockEnhancements;

// Exit if accessed directly.
defined( 'ABSPATH' ) || exit;

if ( ! class_exists( 'Style' ) ) :
	/**
	 * The controller class for style.
	 */
	class Style extends CoreComponent {
		/**
		 * An array of features
		 *
		 * @var array
		 */
		private $features = [];

		/**
		 * An array of block name
		 *
		 * @var array
		 */
		private $block_names = [];

		/**
		 * An array of block names by feature
		 *
		 * @var array
		 */
		private $block_name_by_features = [];

		/**
		 * The constructor
		 */
		public function __construct( $the_plugin_instance ) {
			parent::__construct( $the_plugin_instance );

			// Build features and block names.
			$this->build_features_block_names( $this->the_plugin_instance->get_features() );
		}

		/**
		 * Build list of features, blockNames
		 *
		 * @param array $features
		 * @return void
		 */
		private function build_features_block_names( $features ) {
			// Get an array of supported features.
			$this->features = array_keys( $features );

			// Get an array of supported block names.
			foreach ( $features as $feature_name => $value_by_feature ) {
				if ( $value_by_feature['allowedBlocks'] ?? false ) {
					$this->block_name_by_features[ $feature_name ] = [];
					foreach ( $value_by_feature['allowedBlocks'] as $item ) {
						$this->block_name_by_features[ $feature_name ][] = $item['name'];
						$this->block_names[]                             = $item['name'];
					}
				}
			}

			// Remove duplicated values.
			$this->block_names = array_unique( $this->block_names );
		}

		/**
		 * Run main hooks
		 *
		 * @return void
		 */
		public function run() {
			// Enqueue scripts/styles only for required blocks.
			add_action( 'init', [ $this, 'enqueue_block_style' ] );

			// Render block style.
			add_filter( 'render_block', [ $this, 'render_block_style' ], 10, 2 );

			// Enqueue styles for the iframe editor.
			add_filter( 'block_editor_settings_all', [ $this, 'enqueue_style_for_the_editor' ] );
		}

		/**
		 * Register scripts/styles
		 *
		 * @return void
		 */
		public function enqueue_block_style() {
			// Register with-icon style.
			wp_register_style(
				'block-enhancements-with-icon-styles',
				$this->the_plugin_instance->get_file_uri( 'build/with-icon.css' ),
				[],
				$this->the_plugin_instance->get_script_version()
			);

			// Enqueue feature styles for blocks.
			foreach ( $this->block_name_by_features['withIcon'] as $block_name ) {
				wp_enqueue_block_style(
					$block_name,
					[ 'handle' => 'block-enhancements-with-icon-styles' ]
				);
			}

			// Register with-shadow style.
			wp_register_style(
				'block-enhancements-with-shadow-styles',
				$this->the_plugin_instance->get_file_uri( 'build/with-shadow.css' ),
				[],
				$this->the_plugin_instance->get_script_version()
			);

			// Enqueue feature styles for blocks.
			foreach ( $this->block_name_by_features['withShadow'] as $block_name ) {
				wp_enqueue_block_style(
					$block_name,
					[ 'handle' => 'block-enhancements-with-shadow-styles' ]
				);
			}

			// Register with-text-shadow style.
			wp_register_style(
				'block-enhancements-with-text-shadow-styles',
				$this->the_plugin_instance->get_file_uri( 'build/with-text-shadow.css' ),
				[],
				$this->the_plugin_instance->get_script_version()
			);

			// Enqueue feature styles for blocks.
			foreach ( $this->block_name_by_features['withTextShadow'] as $block_name ) {
				wp_enqueue_block_style(
					$block_name,
					[ 'handle' => 'block-enhancements-with-text-shadow-styles' ]
				);
			}

			// Register with-transform style.
			wp_register_style(
				'block-enhancements-with-transform-styles',
				$this->the_plugin_instance->get_file_uri( 'build/with-transform.css' ),
				[],
				$this->the_plugin_instance->get_script_version()
			);

			// Enqueue feature styles for blocks.
			foreach ( $this->block_name_by_features['withTransform'] as $block_name ) {
				wp_enqueue_block_style(
					$block_name,
					[ 'handle' => 'block-enhancements-with-transform-styles' ]
				);
			}

			// Register with-text-alignment style.
			wp_register_style(
				'block-enhancements-with-text-alignment-styles',
				$this->the_plugin_instance->get_file_uri( 'build/with-text-alignment.css' ),
				[],
				$this->the_plugin_instance->get_script_version()
			);

			// Enqueue feature styles for blocks.
			foreach ( $this->block_name_by_features['withTextAlignment'] as $block_name ) {
				wp_enqueue_block_style(
					$block_name,
					[ 'handle' => 'block-enhancements-with-text-alignment-styles' ]
				);
			}
		}

		/**
		 * Enqueue scripts/styles for the editor
		 *
		 * @param array $editor_settings
		 * @return void
		 */
		public function enqueue_style_for_the_editor( $editor_settings ) {
			// With icon style file path.
			$with_icon_style_file = $this->the_plugin_instance->get_file_path( 'build/with-icon.css' );
			if ( \file_exists( $with_icon_style_file ) ) {
				$with_icon_style = \file_get_contents( $with_icon_style_file );

				if ( $with_icon_style ) {
					$editor_settings['styles'][] = [ 'css' => $with_icon_style ];
				}
			}

			// With shadow style file path.
			$with_shadow_style_file = $this->the_plugin_instance->get_file_path( 'build/with-shadow.css' );
			if ( \file_exists( $with_shadow_style_file ) ) {
				$with_shadow_style = \file_get_contents( $with_shadow_style_file );

				if ( $with_shadow_style ) {
					$editor_settings['styles'][] = [ 'css' => $with_shadow_style ];
				}
			}

			// With text shadow style file path.
			$with_text_shadow_style_file = $this->the_plugin_instance->get_file_path( 'build/with-text-shadow.css' );
			if ( \file_exists( $with_text_shadow_style_file ) ) {
				$with_text_shadow_style = \file_get_contents( $with_text_shadow_style_file );

				if ( $with_text_shadow_style ) {
					$editor_settings['styles'][] = [ 'css' => $with_text_shadow_style ];
				}
			}

			// With transform style file path.
			$with_transform_style_file = $this->the_plugin_instance->get_file_path( 'build/with-transform.css' );
			if ( \file_exists( $with_transform_style_file ) ) {
				$with_transform_style = \file_get_contents( $with_transform_style_file );

				if ( $with_transform_style ) {
					$editor_settings['styles'][] = [ 'css' => $with_transform_style ];
				}
			}

			// With text alignment style file path.
			$with_text_alignment_style_file = $this->the_plugin_instance->get_file_path( 'build/with-text-alignment.css' );
			if ( \file_exists( $with_text_alignment_style_file ) ) {
				$with_text_alignment_style = \file_get_contents( $with_text_alignment_style_file );

				if ( $with_text_alignment_style ) {
					$editor_settings['styles'][] = [ 'css' => $with_text_alignment_style ];
				}
			}

			return $editor_settings;
		}

		/**
		 * Render style for all supported blocks
		 *
		 * @param string $block_content
		 * @param array  $block
		 * @return string
		 */
		public function render_block_style( $block_content, $block ) {
			// Bail if the block has no style.
			if ( ! $this->has_style( $block ) ) {
				return $block_content;
			}

			// Buil selector.
			$selector = $this->generate_selector( $block['blockName'], '' );

			// Get custom style.
			$block_style = $this->get_block_style( $block, '.' . $selector );
			if ( empty( $block_style ) ) {
				return $block_content;
			}

			// Add block class.
			$block_class = str_replace( '/', '-', $block['blockName'] );

			// Add selector to block wrapper element.
			$content = $this->add_class_to_block( $block_content, "{$block_class} {$selector}" );

			return $content . '<style>' . $block_style . '</style>';
		}

		/**
		 * Check whether current block has custom style or not.
		 *
		 * @param [type] $block
		 * @return boolean
		 */
		private function has_style( $block ) {
			// There is no boldblocks value.
			if ( ! isset( $block['attrs']['boldblocks'] ) ) {
				return false;
			}

			// There is no feature in supported features.
			if ( ! array_intersect( $this->features, array_keys( $block['attrs']['boldblocks'] ) ) ) {
				return false;
			}

			return true;
		}

		/**
		 * Check whether current block supports feature
		 *
		 * @param string $feature_name
		 * @param string $block_name
		 * @return boolean
		 */
		private function has_feature( $feature_name, $block_name ) {
			return in_array( $block_name, $this->block_name_by_features[ $feature_name ], true );
		}

		/**
		 * Get block custom style
		 *
		 * @param array  $block
		 * @param string $selector
		 * @return string
		 */
		private function get_block_style( $block, $selector ) {
			$style = '';

			$style_array            = [];
			$responsive_style_array = [];

			// Get responsive settings.
			$breakpoints = get_option( 'be_breakpoints' );
			if ( empty( $breakpoints ) ) {
				$breakpoints = [
					[
						'sm' => [
							'breakpoint' => '576px',
							'mediaQuery' => '',
						],
						'md' => [
							'breakpoint' => '768px',
							'mediaQuery' => '@media (min-width: 768px){##CONTENT##}',
						],
						'lg' => [
							'breakpoint' => '1024px',
							'mediaQuery' => '@media (min-width: 1024px){##CONTENT##}',
						],
					],
				];
			}

			// With icon.
			$this->get_with_icon_style(
				[
					'data'     => $block['attrs']['boldblocks'],
					'selector' => $selector,
					'block'    => $block,
				],
				$style_array
			);

			// With shadow.
			$this->get_with_shadow_style(
				[
					'data'     => $block['attrs']['boldblocks'],
					'selector' => $selector,
					'block'    => $block,
				],
				$style_array
			);

			// With text shadow.
			$this->get_with_text_shadow_style(
				[
					'data'     => $block['attrs']['boldblocks'],
					'selector' => $selector,
					'block'    => $block,
				],
				$style_array
			);

			// With transform.
			$this->get_with_transform_style(
				[
					'data'        => $block['attrs']['boldblocks'],
					'selector'    => $selector,
					'block'       => $block,
					'breakpoints' => $breakpoints,
				],
				$style_array,
				$responsive_style_array
			);

			// With text alignment.
			$this->get_with_text_alignment_style(
				[
					'data'        => $block['attrs']['boldblocks'],
					'selector'    => $selector,
					'block'       => $block,
					'breakpoints' => $breakpoints,
				],
				$style_array,
				$responsive_style_array
			);

			if ( $style_array ) {
				foreach ( $style_array as $selector_type => $value ) {
					if ( count( $value ) > 0 ) {
						$style .= $selector_type . '{' . implode( '', $value ) . '}';
					}
				}
			}

			if ( $responsive_style_array ) {
				foreach ( $responsive_style_array as $responsive_style ) {
					$style .= $responsive_style;
				}
			}

			return $style;
		}

		/**
		 * Build icon style
		 *
		 * @param array  $args {
		 *   @param array  $data
		 *   @param string $sesector
		 *   @param array  $block
		 *   @param array  $devices
		 * }
		 * @param array  &$style_array
		 * @return void
		 */
		private function get_with_icon_style( $args, &$style_array ) {
			$data = $args['data'] ?? [];

			// Block name.
			$block_name = $args['block']['blockName'] ?? '';

			// Has no feature.
			if ( ! $this->has_feature( 'withIcon', $block_name ) ) {
				return '';
			}

			// No value.
			if ( ! isset( $data['withIcon'] ) || ! isset( $data['withIcon']['iconURI'] ) ) {
				return '';
			}

			// Get selector.
			$selector = $args['selector'] ?? '';

			if ( 'core/button' === $block_name ) {
				$selector = $selector . ' .wp-block-button__link';
			} elseif ( 'core/list' === $block_name ) {
				$selector = $selector . ' > li';
			}

			// Custom style.
			$this->build_custom_style(
				array_merge(
					$args,
					[
						'selector'         => $selector,
						'setting_value'    => $data['withIcon'],
						'func_build_style' => [ $this, 'build_with_icon_style' ],
					]
				),
				$style_array
			);
		}

		/**
		 * Build icon style
		 *
		 * @param array $args
		 * @param array &$style_array
		 * @return void
		 */
		private function build_with_icon_style( $args, &$style_array ) {
			$selector      = $args['selector'];
			$setting_value = $args['setting_value'];

			$icon_style   = [];
			$nested_style = [];
			$item_style   = [];

			$icon_uri          = $setting_value['iconURI'] ?? false;
			$icon_width        = $setting_value['iconWidth']['value'] ?? '1em';
			$icon_text_spacing = $setting_value['iconTextSpacing']['value'] ?? '.5em';
			$icon_position     = $setting_value['iconPosition'] ?? 'left';
			$text_alignment    = $setting_value['textAlignment'] ?? false;
			$icon_margin_top   = $setting_value['iconMarginTop'] ?? false;
			$icon_color        = $setting_value['iconColor'] ?? false;

			if ( $icon_uri ) {
				// IconURI.
				$icon_style[] = "--be--with-icon--uri:url(\"{$icon_uri}\");";

				// Icon width.
				$icon_style[] = "--be--with-icon--width:{$icon_width};";

				// Grid column.
				$icon_grid_column = 'left' === $icon_position ? 1 : 2;
				$icon_style[]     = "--be--with-icon--icon-column:{$icon_grid_column};";

				// Margin top.
				if ( $icon_margin_top && isset( $icon_margin_top['value'] ) ) {
					$icon_style[] = "--be--with-icon--mt:{$icon_margin_top['value']};";
				}

				// Icon color.
				$icon_css_color = $icon_color ? $this->get_css_color_value( $icon_color ) : 'currentColor';
				$icon_style[]   = "--be--with-icon--color:{$icon_css_color};";

				// Text alignment.
				if ( $text_alignment ) {
					$item_style[] = "--be--with-icon--text-align:{$text_alignment};";
				}

				// Grid template columns.
				$grid_template_columns = 'left' === $icon_position ? $icon_width . ' 1fr' : '1fr ' . $icon_width;
				$item_style[]          = "--be--with-icon--gtc:{$grid_template_columns};";

				// Spacing.
				$item_style[] = "--be--with-icon--spacing:{$icon_text_spacing};";

				// Nested variables.
				$text_grid_column = 'left' === $icon_position ? 2 : 1;
				$nested_style[]   = "--be--with-icon--text-column:{$text_grid_column};";

				// Add style to the style_array.
				$this->add_to_style_array( implode( '', $item_style ), $style_array, $args );
				$this->add_to_style_array( implode( '', $nested_style ), $style_array, array_merge( $args, [ 'selector' => "{$selector} > * " ] ) );
				$this->add_to_style_array( implode( '', $icon_style ), $style_array, array_merge( $args, [ 'selector' => "{$selector}::before" ] ) );
			}
		}

		/**
		 * Build shadow style
		 *
		 * @param array  $args {
		 *   @param array  $data
		 *   @param string $sesector
		 *   @param array  $block
		 *   @param array  $devices
		 * }
		 * @param array  &$style_array
		 * @return void
		 */
		private function get_with_shadow_style( $args, &$style_array ) {
			$data = $args['data'] ?? [];

			// Block name.
			$block_name = $args['block']['blockName'] ?? '';

			// Has no feature.
			if ( ! $this->has_feature( 'withShadow', $block_name ) ) {
				return '';
			}

			// No value.
			if ( ! isset( $data['withShadow'] ) || ! isset( $data['withShadow']['shadows'] ) ) {
				return '';
			}

			// Get selector.
			$selector = $args['selector'] ?? '';

			if ( 'core/button' === $block_name ) {
				$selector = $selector . ' .wp-block-button__link';
			}

			// Custom style.
			$this->build_custom_style(
				array_merge(
					$args,
					[
						'selector'         => $selector,
						'setting_value'    => $data['withShadow'],
						'func_build_style' => [ $this, 'build_with_shadow_style' ],
					]
				),
				$style_array
			);
		}

		/**
		 * Build shadow style
		 *
		 * @param array $args
		 * @param array &$style_array
		 * @return void
		 */
		private function build_with_shadow_style( $args, &$style_array ) {
			$setting_value = $args['setting_value'];

			// Get shadow value.
			$shadows = $setting_value['shadows'] ?? [];

			$shadow_styles = [];
			if ( is_array( $shadows ) && ! empty( $shadows ) ) {
				foreach ( $shadows as $shadow ) {
					$color = $this->get_css_color_value( $shadow['color'] ?? '' );
					if ( $color ) {
						$inset           = $shadow['inset'] ? 'inset ' : '';
						$x               = $shadow['x'] ?? '0px';
						$y               = $shadow['y'] ?? '0px';
						$blur            = $shadow['blur'] ?? '0px';
						$spread          = $shadow['spread'] ?? '0px';
						$shadow_styles[] = "{$inset}{$x} {$y} {$blur} {$spread} {$color}";
					}
				}
			}

			// Add style to the style_array.
			if ( count( $shadow_styles ) > 0 ) {
				$shadow_css = '--be--box-shadow:' . implode( ',', $shadow_styles ) . ';';
				$this->add_to_style_array( $shadow_css, $style_array, $args );
			}
		}

		/**
		 * Build text shadow style
		 *
		 * @param array  $args {
		 *   @param array  $data
		 *   @param string $sesector
		 *   @param array  $block
		 *   @param array  $devices
		 * }
		 * @param array  &$style_array
		 * @return void
		 */
		private function get_with_text_shadow_style( $args, &$style_array ) {
			$data = $args['data'] ?? [];

			// Block name.
			$block_name = $args['block']['blockName'] ?? '';

			// Has no feature.
			if ( ! $this->has_feature( 'withTextShadow', $block_name ) ) {
				return '';
			}

			// No value.
			if ( ! isset( $data['withTextShadow'] ) || ! isset( $data['withTextShadow']['shadows'] ) ) {
				return '';
			}

			// Get selector.
			$selector = $args['selector'] ?? '';

			// Custom style.
			$this->build_custom_style(
				array_merge(
					$args,
					[
						'selector'         => $selector,
						'setting_value'    => $data['withTextShadow'],
						'func_build_style' => [ $this, 'build_with_text_shadow_style' ],
					]
				),
				$style_array
			);
		}

		/**
		 * Build text shadow style
		 *
		 * @param array $args
		 * @param array &$style_array
		 * @return void
		 */
		private function build_with_text_shadow_style( $args, &$style_array ) {
			$setting_value = $args['setting_value'];

			// Get shadow value.
			$shadows = $setting_value['shadows'] ?? [];

			$shadow_styles = [];
			if ( is_array( $shadows ) && ! empty( $shadows ) ) {
				foreach ( $shadows as $shadow ) {
					$color = $this->get_css_color_value( $shadow['color'] ?? '' );
					if ( $color ) {
						$x               = $shadow['x'] ?? '0px';
						$y               = $shadow['y'] ?? '0px';
						$blur            = $shadow['blur'] ?? '0px';
						$shadow_styles[] = "{$x} {$y} {$blur} {$color}";
					}
				}
			}

			// Add style to the style_array.
			if ( count( $shadow_styles ) > 0 ) {
				$shadow_css = '--be--text-shadow:' . implode( ',', $shadow_styles ) . ';';
				$this->add_to_style_array( $shadow_css, $style_array, $args );
			}
		}

		/**
		 * Build transform style
		 *
		 * @param array  $args {
		 *   @param array  $data
		 *   @param string $sesector
		 *   @param array  $block
		 *   @param array  $devices
		 * }
		 * @param array  &$style_array
		 * @param array  &$responsive_style_array
		 * @return void
		 */
		private function get_with_transform_style( $args, &$style_array, &$responsive_style_array ) {
			$data = $args['data'] ?? [];

			// Block name.
			$block_name = $args['block']['blockName'] ?? '';

			// Has no feature.
			if ( ! $this->has_feature( 'withTransform', $block_name ) ) {
				return '';
			}

			// No value.
			if ( ! isset( $data['withTransform'] ) || ! isset( $data['withTransform']['transform'] ) ) {
				return '';
			}

			// Get selector.
			$selector = $args['selector'] ?? '';

			if ( 'core/button' === $block_name ) {
				$selector = $selector . ' .wp-block-button__link';
			}

			// Custom style.
			$this->build_custom_style(
				array_merge(
					$args,
					[
						'selector'         => $selector,
						'setting_value'    => $data['withTransform'],
						'func_build_style' => [ $this, 'build_with_transform_style' ],
					]
				),
				$style_array,
				$responsive_style_array
			);
		}

		/**
		 * Build transform style
		 *
		 * @param array $args
		 * @param array &$style_array
		 * @param array &$responsive_style_array
		 * @return void
		 */
		private function build_with_transform_style( $args, &$style_array, &$responsive_style_array ) {
			$setting_value = $args['setting_value'];

			// Get transform value.
			$transform = $setting_value['transform'] ?? [];

			// Responsive style.
			$transform_style = $this->build_responsive_style(
				array_merge(
					$args,
					[
						'setting_value'    => $transform,
						'setting_variable' => '--be--transform',
						'func_build_style' => [ $this, 'build_transform_css_value' ],
					]
				),
				$style_array,
				$responsive_style_array
			);

			// Add style to the style_array.
			if ( $transform_style ) {
				$transform_origin = $setting_value['transformOrigin'] ?? [];
				$x                = $transform_origin['x'] ?? .5;
				if ( ! \is_numeric( $x ) ) {
					$x = .5;
				}
				$x = round( $x, 2 ) * 100;
				$y = $transform_origin['y'] ?? .5;
				if ( ! \is_numeric( $y ) ) {
					$y = .5;
				}
				$y = round( $y, 2 ) * 100;

				$transform_origin_css = "--be--transform--origin: {$x}% {$y}%;";
				$this->add_to_style_array( $transform_origin_css, $style_array, $args );
			}
		}

		/**
		 * Build CSS value for transform
		 *
		 * @param array $args
		 * @return string
		 */
		private function build_transform_css_value( $args = [] ) {
			$value = $args['value'] ?? [];
			if ( ! empty( $value ) && is_array( $value ) ) {
				return implode( ' ', array_filter( array_map( [ $this, 'build_transform_item' ], $value ) ) );
			}

			return '';
		}

		/**
		 * Build transform item: translate, rotate, scale, skew
		 *
		 * @return string
		 */
		private function build_transform_item( $transform_item ) {
			$style = '';
			if ( is_array( $transform_item ) && count( $transform_item ) > 0 ) {
				// Get tranform type.
				$transform_type  = array_keys( $transform_item )[0];
				$transform_value = $transform_item[ $transform_type ] ?? '';

				if ( $transform_type === 'rotate' ) {
					if ( is_numeric( $transform_value ) ) {
						$style = "rotate({$transform_value}deg)";
					}
				} elseif ( is_array( $transform_value ) ) {
					$suffix = 'skew' === $transform_type ? 'deg' : '';

					$x       = $transform_value['x'] ?? '';
					$y       = $transform_value['y'] ?? '';
					$x_value = preg_replace( '/[^0-9.]/', '', $x );
					$y_value = preg_replace( '/[^0-9.]/', '', $y );

					if ( \is_numeric( $x_value ) || \is_numeric( $y_value ) ) {
						if ( ! \is_numeric( $x_value ) ) {
							if ( 'scale' === $transform_type ) {
								$x = 1;
							} else {
								$x = 0;
							}
						}

						if ( ! \is_numeric( $y_value ) ) {
							if ( 'scale' === $transform_type ) {
								$y = 1;
							} else {
								$y = 0;
							}
						}

						$style = "{$transform_type}({$x}{$suffix}, {$y}{$suffix})";
					}
				}
			}

			return $style;
		}

		/**
		 * Build text alignment style
		 *
		 * @param array  $args {
		 *   @param array  $data
		 *   @param string $sesector
		 *   @param array  $block
		 *   @param array  $devices
		 * }
		 * @param array  &$style_array
		 * @param array  &$responsive_style_array
		 * @return void
		 */
		private function get_with_text_alignment_style( $args, &$style_array, &$responsive_style_array ) {
			$data = $args['data'] ?? [];

			// Block name.
			$block_name = $args['block']['blockName'] ?? '';

			// Has no feature.
			if ( ! $this->has_feature( 'withTextAlignment', $block_name ) ) {
				return '';
			}

			// No value.
			if ( ! isset( $data['withTextAlignment'] ) || ! isset( $data['withTextAlignment']['textAlignment'] ) ) {
				return '';
			}

			// Responsive style.
			$this->build_responsive_style(
				array_merge(
					$args,
					[
						'setting_value'    => $data['withTextAlignment']['textAlignment'],
						'setting_variable' => '--be--text-alignment',
						'func_build_style' => [ $this, 'build_text_alignment_css_value' ],
					]
				),
				$style_array,
				$responsive_style_array
			);
		}

		/**
		 * Build text alignment style
		 *
		 * @param array $args
		 * @return void
		 */
		private function build_text_alignment_css_value( $args ) {
			if ( in_array( $args['value'], [ 'left', 'center', 'right' ] ) ) {
				return $args['value'];
			}

			return '';
		}

		/**
		 * Build custom style
		 *
		 * @param array $args
		 * @param array &$style_array
		 * @param array &$responsive_style_array
		 * @return string
		 */
		private function build_custom_style( $args, &$style_array = [], &$responsive_style_array = [] ) {
			$func_build_style = $args['func_build_style'] ?? '';

			if ( ! \is_callable( $func_build_style ) ) {
				return '';
			}

			$setting_value = $args['setting_value'] ?? [];
			if ( empty( $setting_value ) ) {
				return '';
			}

			// Build style.
			return $func_build_style( $args, $style_array, $responsive_style_array );
		}

		/**
		 * Build responsive style
		 *
		 * @param array $args
		 * @param array &$style_array
		 * @param array &$responsive_style_array
		 * @return string
		 */
		private function build_responsive_style( $args = [], &$style_array, &$responsive_style_array ) {
			$func_build_style = $args['func_build_style'] ?? '';
			if ( ! \is_callable( $func_build_style ) ) {
				return '';
			}

			$setting_value = $args['setting_value'] ?? [];
			if ( empty( $setting_value ) || ! \is_array( $setting_value ) ) {
				return '';
			}

			$setting_variable = $args['setting_variable'] ?? [];
			if ( empty( $setting_variable ) ) {
				return '';
			}

			$breakpoints = $args['breakpoints'];
			$selector    = $args['selector'];

			$setting_styles = [];
			if ( is_array( $setting_value ) && ! empty( $setting_value ) ) {
				foreach ( $setting_value as $breakpoint => $value_by_breakpoint ) {
					$value = [];
					if ( isset( $value_by_breakpoint['value'] ) ) {
						$value = $value_by_breakpoint['value'];
					} elseif ( ! isset( $value_by_breakpoint['value'] ) && isset( $value_by_breakpoint['inherit'] ) ) {
						$value = $value_by_breakpoint['inherit']['value'] ?? '';
					}

					$style_by_breakpoint = $func_build_style( array_merge( $args, [ 'value' => $value ] ) );
					if ( $style_by_breakpoint ) {
						$setting_styles[ $breakpoint ] = "{$setting_variable}:{$style_by_breakpoint};";
					}
				}
			}

			$style      = '';
			$last_style = '';
			foreach ( $setting_styles as $breakpoint => $style_by_breakpoint ) {
				if ( $style_by_breakpoint !== $last_style ) {
					$style_with_selector = "{$selector}{{$style_by_breakpoint}}";
					$media_query         = $breakpoints[ $breakpoint ]['mediaQuery'] ?? '';
					if ( $media_query ) {
						$style .= \str_replace( '##CONTENT##', $style_with_selector, $media_query );
					} else {
						$style .= $style_with_selector;
					}
					$last_style = $style_by_breakpoint;
				}
			}

			// Add style to the style_array.
			if ( $style ) {
				$responsive_style_array[] = $style;
			}

			return $style;
		}

		/**
		 * Add style to the style array
		 *
		 * @param string $style
		 * @param array  &$style_array
		 * @param array  $args
		 * @return void
		 */
		private function add_to_style_array( $style, &$style_array, $args ) {
			if ( $style ) {
				$selector = $args['selector'];

				if ( ! isset( $style_array[ $selector ] ) ) {
					$style_array[ $selector ] = [];
				}

				$style_array[ $selector ][] = $style;
			}
		}

		/**
		 * Check wether the block markup has a attribute or not.
		 *
		 * @param string $attribute
		 * @param string $block_content
		 * @return boolean
		 */
		private function block_has_attribute( $attribute, $block_content ) {
			$greater_position = \strpos( $block_content, '>' );
			if ( $greater_position !== false ) {
				$block_tag = \substr( $block_content, 0, $greater_position + 1 );

				return \strpos( $block_tag, $attribute . '=' );
			}

			return false;
		}

		/**
		 * Generate block selector.
		 *
		 * @param string $block_name
		 * @param string $prefix
		 * @return string
		 */
		private function generate_selector( $block_name, $prefix = '' ) {
			// Generate an unique value.
			$id              = \uniqid();
			$selector_prefix = str_replace( '/', '-', $block_name );

			// Buil selector.
			return "{$prefix}{$selector_prefix}-{$id}";
		}

		/**
		 * Add CSS class to block wrapper
		 *
		 * @param string $block_content
		 * @param string $classes
		 * @return string
		 */
		private function add_class_to_block( $block_content, $classes ) {
			if ( ! $this->block_has_attribute( 'class', $block_content ) ) {
				$content = \preg_replace(
					'/' . \preg_quote( '>', '/' ) . '/',
					'class="' . $classes . '">',
					$block_content,
					1
				);
			} else {
				$content = \preg_replace(
					'/' . \preg_quote( 'class="', '/' ) . '/',
					'class = "' . $classes . ' ',
					$block_content,
					1
				);
			}

			return $content;
		}

		/**
		 * Get CSS value for a color object.
		 *
		 * @param array $color_array
		 * @return string
		 */
		private function get_css_color_value( $color_array ) {
			$value = '';

			if ( is_array( $color_array ) && ! empty( $color_array ) ) {
				if ( $color_array['gradientSlug'] ?? false ) {
					$value = "var(--wp--preset--gradient--{$color_array['gradientSlug']}, {$color_array['gradientValue']})";
				} elseif ( $color_array['gradientValue'] ?? false ) {
					$value = $color_array['gradientValue'];
				} elseif ( $color_array['slug'] ?? false ) {
					$value = "var(--wp--preset--color--{$color_array['slug']}, {$color_array['value']})";
				} elseif ( $color_array['value'] ?? false ) {
					$value = $color_array['value'];
				}
			}

			return $value;
		}
	}
endif;
